# Licensed under the MIT License
# https://github.com/craigahobbs/simple-git-changelog/blob/main/LICENSE

import re
import os

from setuptools import setup

MODULE_NAME = 'simple_git_changelog'
PACKAGE_NAME = 'simple-git-changelog'

def main():
    # Read the package version
    with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'src', MODULE_NAME, '__init__.py'), encoding='utf-8') as init_file:
        version = re.search(r"__version__ = '(.+?)'", init_file.read()).group(1)

    # Read the readme for use as the long description
    with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'README.md'), encoding='utf-8') as readme_file:
        long_description = readme_file.read()

    # Do the setup
    setup(
        name=PACKAGE_NAME,
        description='Simple git changelog file generator',
        long_description=long_description,
        long_description_content_type='text/markdown',
        version=version,
        author='Craig Hobbs',
        author_email='craigahobbs@gmail.com',
        keywords='git changelog',
        url='https://github.com/craigahobbs/' + PACKAGE_NAME,
        license='MIT',
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.10',
            'Topic :: Utilities'
        ],
        package_dir={'': 'src'},
        packages=[MODULE_NAME],
        entry_points={
            'console_scripts': [
                PACKAGE_NAME + ' = ' + MODULE_NAME + '.main:main'
            ],
        }
    )

if __name__ == '__main__':
    main()



# Licensed under the MIT License
# https://github.com/craigahobbs/simple-git-changelog/blob/main/LICENSE

import argparse
from datetime import date
import os
import re
import subprocess

from . import __version__ as VERSION


def main(args=None):

    # Command-line arguments
    parser = argparse.ArgumentParser(prog='simple-git-changelog')
    parser.add_argument('-o', metavar='FILE', dest='output', default='CHANGELOG.md',
                        help='specify the change log file (default is "CHANGELOG.md")')
    parser.add_argument('--version', action='store_true',
                        help='show version number and quit')
    args = parser.parse_args(args=args)
    if args.version:
        parser.exit(message=VERSION + '\n')

    # Parse the change log file, if any
    if os.path.isfile(args.output):
        with open(args.output, 'r') as changelog_file:
            changelog_lines = list(changelog_file)
        changelog_commits = parse_changelog(changelog_lines)
    else:
        changelog_commits = set()
        changelog_lines = []

    # Any new changes?
    git_changes = get_git_changes()
    if git_changes and git_changes[0][0] not in changelog_commits:
        git_url = get_git_url()

        # Write the updated changelog file
        with open(args.output, 'w') as changelog_file:

            # Write entries for new changes
            changelog_file.write(f'## {date.today().isoformat()}\n')
            for commit, subject in git_changes:
                if commit in changelog_commits:
                    break
                changelog_file.write(f'\n- [{commit}]({git_url}/commit/{commit}) - {subject}\n')

            # Write pre-existing changelog lines
            if changelog_lines:
                changelog_file.write('\n')
            for changelog_line in changelog_lines:
                changelog_file.write(changelog_line)


def parse_changelog(lines):
    re_change = re.compile(r'^-\s+\[(?P<commit>[0-9a-f]+)\]')
    commits = set()
    for line in lines:
        change_match = re_change.search(line)
        if change_match is not None:
            commits.add(change_match.group('commit'))
    return commits


def get_git_changes():
    commits = subprocess.check_output(['git', 'log', '--pretty=format:%h %s'], encoding='utf-8')
    return [line.split(' ', 1) for line in commits.splitlines()]


def get_git_url():
    git_origin = subprocess.check_output(['git', 'config', '--get', 'remote.origin.url'], encoding='utf-8').strip()
    git_url = re.sub(r'\.git$', '', git_origin)
    git_url_match = re.search(r'^git@(?P<domain>[^:]+):(?P<user>[^/]+)/(?P<project>.+)$', git_url)
    if git_url_match is not None:
        git_url = f'https://{git_url_match.group("domain")}/{git_url_match.group("user")}/{git_url_match.group("project")}'
    return git_url
